package com.gree.mall.contest.controller.pc;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.Freight.FreightTemplateBean;
import com.gree.mall.contest.bean.Freight.RegionTreeBean;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.FreightLogic;
import com.gree.mall.contest.plus.entity.FreightTemplate;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Slf4j
@RestController
@Tag(name = "运费管理", description = "运费管理")
@RequestMapping(value = "/pc/freight", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class FreightController {
    
    private final FreightLogic freightLogic;

    @GetMapping("/list")
    @Operation(summary = "运费模板列表")
    public ResponseHelper<Page<FreightTemplate>> page(
            HttpServletRequest request,
            @Parameter(description = "名称",required = false) @RequestParam(required = false) String name,
            @Parameter(description = "状态(true：启用 false:禁用",required = false) @RequestParam(required = false) Boolean status,
            @Parameter(description = "开始时间",required = false) @RequestParam(required = false) String startTime,
            @Parameter(description = "结束时间",required = false) @RequestParam(required = false) String endTime,
            @Parameter(description = "页号",required = true) @RequestParam(required = true) Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam(required = true) Integer pageSize
    ){
        IPage<FreightTemplate> list = freightLogic.list(request,name, status, startTime, endTime, pageNum, pageSize);
        return ResponseHelper.success(list);
    }

    @GetMapping("/detail")
    @Operation(summary = "运费模板详情")
    public ResponseHelper<FreightTemplateBean> detail(
            @Parameter(description = "模板id",required = true) @RequestParam(required = true) String freightTemplateId
    ){
        FreightTemplateBean detail = freightLogic.detail(freightTemplateId);
        return ResponseHelper.success(detail);
    }


    @PostMapping("/delete")
    @Operation(summary = "删除运费模板")
    public ResponseHelper delete(
            @Parameter(description = "模板id",required = true) @RequestParam(required = true) String freightTemplateId
    ){
        freightLogic.delete(freightTemplateId);
        return ResponseHelper.success();
    }

    @PostMapping("/save")
    @Operation(summary = "新增修改运费模板")
    public ResponseHelper delete(
            @Parameter(description = "模板",required = true) @RequestBody(required = true) FreightTemplateBean freightTemplateBean,
            HttpServletRequest request
    ){
        freightLogic.save(freightTemplateBean,request);
        return ResponseHelper.success();
    }

    @PostMapping("/status/update")
    @Operation(summary = "修改运费模板状态")
    public ResponseHelper updateStatus(
            @Parameter(description = "模板id",required = true) @RequestParam(required = true) String freightTemplateId,
            @Parameter(description = "状态:true=启用 false=禁用",required = true) @RequestParam(required = true) Boolean status
    ){
        freightLogic.updateStatus(freightTemplateId,status);
        return ResponseHelper.success();
    }


    @GetMapping("/region/tree")
    @Operation(summary = "获取地区树")
    public ResponseHelper<List<RegionTreeBean>> regionTree(
            @Parameter(description = "等级(1=省，2=市，3=区，4=街道，5=社区)(默认为区以上)",required = false) @RequestParam(required = false) Integer level
    ){
        List<RegionTreeBean> regionTreeBeans = freightLogic.regionTree(level);
        return ResponseHelper.success(regionTreeBeans);
    }

}
