package com.gree.mall.contest.controller.pc;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.StorageLogic;
import com.gree.mall.contest.plus.entity.Storage;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

@Slf4j
@RestController
@Tag(name = "仓储管理", description = "仓储管理")
@RequestMapping(value = "/pc/storage", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class StorageController {
    
    private final StorageLogic storageLogic;

    @GetMapping("/list")
    @Operation(summary = "仓储列表")
    public ResponseHelper<Page<Storage>> page(
            @Parameter(description = "页号",required = true) @RequestParam Integer pageNum,
            @Parameter(description = "页大小",required = true) @RequestParam Integer pageSize,
            HttpServletRequest request
    ) throws RemoteServiceException {
        IPage<Storage> page = storageLogic.page(pageNum, pageSize,request);
        return ResponseHelper.success(page);
    }

    @GetMapping("/detail")
    @Operation(summary = "仓储详情")
    public ResponseHelper<Storage> detail(
            @Parameter(description = "id",required = true) @RequestParam() String id
    ) throws RemoteServiceException {
        Storage detail = storageLogic.detail(id);
        return ResponseHelper.success(detail);
    }


    @PostMapping("/add")
    @Operation(summary = "新增仓储")
    public ResponseHelper add(
            HttpServletRequest request,
            @Parameter(description = "object",required = true) @RequestBody Storage storage) {
        storageLogic.add(request,storage);
        return ResponseHelper.success();
    }

    @PostMapping("/update")
    @Operation(summary = "修改仓储")
    public ResponseHelper update(
            @Parameter(description = "object",required = true) @RequestBody Storage storage) {
        storageLogic.update(storage);
        return ResponseHelper.success();
    }

    @PostMapping("/delete")
    @Operation(summary = "删除仓储")
    public ResponseHelper delete(
            @Parameter(description = "仓储id",required = true) @RequestParam String storageId
    ) {
        storageLogic.delete(storageId);
        return ResponseHelper.success();
    }

}
