package com.gree.mall.contest.controller.pc.apply;


import cn.hutool.core.lang.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.annotation.ApiNotAuth;
import com.gree.mall.contest.annotation.ZfireList;
import com.gree.mall.contest.bean.merchant.WebsitApplyAdd;
import com.gree.mall.contest.bean.merchant.WebsitApplyDetail;
import com.gree.mall.contest.bean.merchant.WebsitApplyVO;
import com.gree.mall.contest.bean.zfire.ZfireParamBean;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.merchant.MerchantLogic;
import com.gree.mall.contest.plus.entity.WebsitApply;
import com.gree.mall.contest.utils.zfire.FieldUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


@Validated
@RequiredArgsConstructor
@RestController
@Tag(name = "商家入驻申请管理,申请入驻服API", description = "商家入驻申请管理,申请入驻API")
@RequestMapping("/pc/websitApply")
public class ApplyController {

    private final MerchantLogic merchantLogic;

    @ApiNotAuth
    @PostMapping("/register")
    @Operation(summary = "注册账号-验证验证码 true下一步 false展示申请审核详情")
    public ResponseHelper<Boolean> register(
            @Parameter(description = "手机号", required = true) @RequestParam String mobile,
            @Parameter(description = "验证码") @RequestParam(required = false) String code
    ) {
        Boolean b = merchantLogic.register(mobile, code);
        return ResponseHelper.success(b);
    }

    @ApiNotAuth
    @PostMapping("/applyMobile")
    @Operation(summary = "展示申请审核详情")
    public ResponseHelper<WebsitApplyDetail> applyMobile(
            @Parameter(description = "手机号", required = true) @RequestParam String mobile
    ) {
        WebsitApplyDetail websitApply = merchantLogic.applyMobile(mobile);
        return ResponseHelper.success(websitApply);
    }

    @ApiNotAuth
    @PostMapping("/applyList")
    @Operation(summary = "申请入驻")
    public ResponseHelper<WebsitApply> applyMobile(
            @RequestBody WebsitApplyAdd websitApplyAdd
    ) {
        WebsitApply websitApply = merchantLogic.applyList(websitApplyAdd);
        return ResponseHelper.success(websitApply);
    }

    @ZfireList
    @PostMapping("/list")
    @Operation(summary = "服务商入驻申请管理-列表")
    public ResponseHelper<IPage<WebsitApplyVO>> pageApply(
            @RequestBody ZfireParamBean zfireParamBean
    ) {
        IPage<WebsitApplyVO> page = merchantLogic.pageApply(zfireParamBean);
        return ResponseHelper.success(page, new TypeReference<WebsitApplyVO>() {
        });
    }

    @PostMapping("/list/export")
    @Operation(summary = "服务商入驻申请管理-导出")
    public void listExport(
            @RequestBody ZfireParamBean zfireParamBean,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        //2.查询要导出的内容
        IPage<WebsitApplyVO> baseVOIPage = merchantLogic.pageApply(zfireParamBean);
        //3.导出
        FieldUtils.exportData(baseVOIPage.getRecords(), zfireParamBean.getExportFields(), request, response);
    }


    @PostMapping("/detail")
    @Operation(summary = "展示申请审核详情")
    public ResponseHelper<WebsitApplyDetail> detail(
            @Parameter(description = "id", required = true) @RequestParam String id
    ) {
        WebsitApplyDetail websitApply = merchantLogic.detail(id);
        return ResponseHelper.success(websitApply);
    }

    @PostMapping("/updateIn")
    @Operation(summary = "跟进")
    public ResponseHelper update(
            @Parameter(description = "id", required = true) @RequestParam String id,
            @Parameter(description = "WAIT 跟进 FAIL 无需跟进", required = true) @RequestParam String status,
            @Parameter(description = "备注", required = true) @RequestParam String remark
    ) {
        merchantLogic.updateIn(id, status, remark);
        return ResponseHelper.success();
    }


}
