package com.gree.mall.contest.controller.pc.store;

import cn.hutool.core.lang.TypeReference;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.gree.mall.contest.annotation.ZfireList;
import com.gree.mall.contest.bean.merchant.MerchantAddBean;
import com.gree.mall.contest.bean.merchant.MerchantBean;
import com.gree.mall.contest.bean.store.PlatformStoreVO;
import com.gree.mall.contest.bean.zfire.ZfireParamBean;
import com.gree.mall.contest.helper.ResponseHelper;
import com.gree.mall.contest.logic.store.PlatformStoreLogic;
import com.gree.mall.contest.plus.entity.PlatformStore;
import com.gree.mall.contest.utils.zfire.FieldUtils;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

@Slf4j
@RestController
@Tag(name = "平台门店管理", description = "平台门店管理")
@RequestMapping(value = "/pc/platform/store", produces = "application/json; charset=utf-8")
@RequiredArgsConstructor
public class PlatformStoreController {

    private final PlatformStoreLogic platformStoreLogic;

    @ZfireList
    @PostMapping("/list")
    @Operation(summary = "列表")
    public ResponseHelper<IPage<PlatformStoreVO>> list(
            @RequestBody ZfireParamBean zfireParamBean
    ) {
        IPage<PlatformStoreVO> page = platformStoreLogic.list(zfireParamBean);
        return ResponseHelper.success(page, new TypeReference<PlatformStoreVO>() {});
    }

    @PostMapping("/list/export")
    @Operation(summary = "导出")
    public void listExport(
            @RequestBody ZfireParamBean zfireParamBean,
            HttpServletRequest request,
            HttpServletResponse response
    ) throws Exception {
        //2.查询要导出的内容
        IPage<PlatformStoreVO> baseVOIPage = platformStoreLogic.list(zfireParamBean);
        //3.导出
        FieldUtils.exportData(baseVOIPage.getRecords(), zfireParamBean.getExportFields(), request, response);
    }

    @PostMapping("/detail")
    @Operation(summary = "详情")
    public ResponseHelper<PlatformStore> detail(
            @RequestParam String id
    ) {
        PlatformStore detail = platformStoreLogic.detail(id);
        return ResponseHelper.success(detail);
    }

    @PostMapping("/add")
    @Operation(summary = "新增")
    public ResponseHelper add(
            @RequestBody PlatformStore platformStore
    ) {
        platformStoreLogic.add(platformStore);
        return ResponseHelper.success();
    }


    @PostMapping("/update")
    @Operation(summary = "修改")
    public ResponseHelper update(
            @RequestBody PlatformStore platformStore
    ) {
        platformStoreLogic.update(platformStore);
        return ResponseHelper.success();
    }
}
