package com.gree.mall.contest.logic.common;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.user.CurrentCompanyWechat;
import com.gree.mall.contest.commonmapper.AdminMapper;
import com.gree.mall.contest.enums.StatusEnum;
import com.gree.mall.contest.enums.admin.RoleTypeEnum;
import com.gree.mall.contest.enums.admin.UpdateLogRemarkEnum;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import com.gree.mall.contest.utils.ApplicationContextUtils;
import com.gree.mall.contest.utils.CommonUtils;
import com.gree.mall.contest.utils.RedisUtil;
import com.gree.mall.contest.utils.oss.OSSUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@RequiredArgsConstructor
public class CommonLogic {

    private static final Logger operationLogger = LoggerFactory.getLogger("OPERATION_LOGGER");

    @Value("${spring.profiles.active}")
    private String active;
    private final WechatLogic wechatLogic;
    private final RedisUtil redisUtil;
    private final AdminCompanyWechatService AdminCompanyWechatService;
    private final CommonFileService commonFileService;
    private final AdminUserService adminUserService;
    private final AdminCompanyWechatService adminCompanyWechatService;
    private final AdminMapper adminMapper;
    private final AdminWebsitService adminWebsitService;
    private final MerchantWebsitService merchantWebsitService;
    private final MerchantWebsitRegionService merchantWebsitRegionService;
    private final UserService userService;
    private final OSSUtil ossUtil;
    private final SysConfigService sysConfigService;
    private final WxSceneService wxSceneService;
    private final UpdateLogLogic updateLogLogic;
    private final LbsAmapService lbsAmapService;
    private final WxSceneDetailService wxSceneDetailService;
    private final RegionService regionService;
    //高德的webkey
    private final String mapWebKey = "69cebe7cded9307a588237b0618856c9";

    public Map<String, String> getOSSConfig() throws UnsupportedEncodingException {
        return ossUtil.getConfig();
    }

    public SysConfig getSysConfig() {
        SysConfig sysConfig = sysConfigService.lambdaQuery().last("limit 1").one();
        if (sysConfig == null) {
            throw new RemoteServiceException("系统缺少必要配置，请联系相关人员");
        }
        return sysConfig;
    }

    /**
     * 查询服务商的所有账号
     *
     * @param serviceProviderId
     * @return
     */
    public List<AdminUser> getAdminUserByServiceProviderId(String serviceProviderId) {
        List<AdminUser> list = adminUserService.lambdaQuery()
                .eq(AdminUser::getStatus, true)
                .eq(AdminUser::getServiceProviderId, serviceProviderId).list();
        return list;
    }


    /**
     * 获取用户信息
     *
     * @param mobile
     * @param flag
     * @return
     */
    public User getUserByMobile(String mobile, Integer flag) {
        if (StringUtils.isBlank(mobile) || flag == null) {
            throw new RemoteServiceException("获取用户信息失败，缺少必要的参数");
        }
        return userService.lambdaQuery()
                .eq(User::getMobile, mobile)
                .one();
    }

    /**
     * 查询当前PC用户
     *
     * @return
     */
    public AdminUserCom getAdminUser() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        return this.getAdminUser(request);
    }

    /**
     * 获取帐号以及部门id集合
     */
    public AdminUserCom getAdminUser(HttpServletRequest request) {
        String userId = CommonUtils.getUserId(request);
        //方便测试用
        if (StringUtils.isBlank(userId) && !active.equals("prd")) {
            AdminUser admin = adminUserService.lambdaQuery().eq(AdminUser::getUserName, "admin").one();
            AdminUserCom adminUserCom = new AdminUserCom();
            BeanUtils.copyProperties(admin, adminUserCom);
            AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById("1");
            adminUserCom.setAdminCompanyWechat(adminCompanyWechat);
            return adminUserCom;
        }
        AdminUser adminUser = adminUserService.getById(userId);
        if (adminUser == null) {
            return null;
        }
        if (!adminUser.getStatus()) {
            throw new RemoteServiceException("帐号已被禁用");
        }
        AdminUserCom adminUserCom = new AdminUserCom();
        BeanUtils.copyProperties(adminUser, adminUserCom);

        //所有帐号
        AdminUserCom account2 = this.websitAccount(adminUserCom, request);

        if (account2 != null) {
            return account2;
        }
        adminUserCom.setCompanyWechatIds(ListUtil.toList("1"));
        // TODO 写死后期处理
        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.getById("1");
        adminUserCom.setAdminCompanyWechat(adminCompanyWechat);
        adminUserCom.setLoginCompanyWechatId(adminCompanyWechat.getCompanyWechatId());
        adminUserCom.setLoginCompanyName(adminCompanyWechat.getCompanyName());
        //暂不过滤权限
        return adminUserCom;
    }

    /**
     * 部门帐号
     */
    public AdminUserCom websitAccount(AdminUserCom adminUserCom, HttpServletRequest request) {
        MerchantWebsit merchantWebsit = merchantWebsitService.getById(adminUserCom.getAdminWebsitId());
        adminUserCom.setMerchantWebsit(merchantWebsit);
        //非平台账号
        if (!adminUserCom.getUserName().equals("admin")) {
            List<MerchantWebsit> list = new ArrayList<>();
            list.addAll(this.queryAllChild(list, adminUserCom.getAdminWebsitId()));
            List<String> websitIds = list.stream().map(MerchantWebsit::getWebsitId).distinct().collect(Collectors.toList());
            websitIds.add(adminUserCom.getAdminWebsitId());
            adminUserCom.setAdminWebsitIds(websitIds.stream().distinct().collect(Collectors.toList()));

            //平台业务员的区id
            if (StringUtils.equals(adminUserCom.getType(), RoleTypeEnum.A.getCode())) {
                //区域id
                List<MerchantWebsitRegion> merchantWebsitRegions = merchantWebsitRegionService.lambdaQuery().in(MerchantWebsitRegion::getAdminWebsitId, adminUserCom.getAdminWebsitIds()).list();
                adminUserCom.setAreaIds(merchantWebsitRegions.stream().map(MerchantWebsitRegion::getAreaId).distinct().collect(Collectors.toList()));
            }

        }

        return adminUserCom;
    }

    public User getUser() {
        HttpServletRequest request = ApplicationContextUtils.getHttpServletRequest();
        String userId = CommonUtils.getUserId(request);
        User user = userService.getById(userId);
        return user;
    }

    /**
     * 递归查询所有数据
     */
    public List<MerchantWebsit> queryAllChild(List<MerchantWebsit> list, String id) {
        if (id == null || id.equals("0")) {
            return list;
        }
        List<MerchantWebsit> merchantWebsits = merchantWebsitService.lambdaQuery().eq(MerchantWebsit::getParentId, id).list();
        if (merchantWebsits.size() == 0) {
            return list;
        }
        for (MerchantWebsit merchantWebsit : merchantWebsits) {
            this.queryAllChild(list, merchantWebsit.getWebsitId());
        }
        list.addAll(merchantWebsits);
        return list;
    }

    /**
     * 上传附件
     *
     * @param file
     * @return
     * @throws IOException
     */
    public CommonFile uploadFile(MultipartFile file) throws IOException, RemoteServiceException {

        if (file == null || file.isEmpty()) {
            return null;
        }
        String uploadFilePath = file.getOriginalFilename().replaceAll("\\\\", "/");
        String uploadFileName = uploadFilePath.substring(
                uploadFilePath.lastIndexOf('/') + 1, uploadFilePath.lastIndexOf('.'));
        String uploadFileSuffix = uploadFilePath.substring(uploadFilePath.lastIndexOf('.') + 1, uploadFilePath.length());
        long currentTime = System.currentTimeMillis();
        BufferedOutputStream bufferedOutputStream = null;

        //限制大小为10m
        if (file.getSize() > 1024 * 1024 * 30) {
            throw new RemoteServiceException("文件大小超出限制");
        }

        try {
            String filePath = ossUtil.getFilePath() + "/" + currentTime + UUID.randomUUID() + "." + uploadFileSuffix;
            if (!ossUtil.uploadFile(filePath, file.getBytes())) {
                throw new RemoteServiceException("文件同步oss失败");
            }
            String ossUrl = ossUtil.getAccessUrl() + filePath;

            CommonFile commonFile = new CommonFile();
            commonFile.setUrl(ossUrl);
            commonFile.setName(uploadFileName);
            commonFile.setFileType(uploadFileSuffix);
            commonFile.setFileSize(BigDecimal.valueOf(file.getSize()));
            commonFile.setCreateTime(new Date());
            commonFileService.save(commonFile);
            return commonFile;
        } finally {
            if (bufferedOutputStream != null) {
                bufferedOutputStream.close();
            }
        }
    }


//    public List<Region> queryRegionList(List<String> pid){
//        return regionService.lambdaQuery().in(Region::getPid,pid).list();
//    }

    public String getFile(String key) throws IOException {
        String url = ossUtil.getUrlWw(key);
        return url;
    }

    public List<LbsAmap> city2(String city) {
        List<LbsAmap> citys = lbsAmapService.lambdaQuery()
                .eq(LbsAmap::getStatus, StatusEnum.ONE.getIntKey())
                .like(LbsAmap::getName, city)
                .eq(LbsAmap::getLevel, "city")
                .list();
        return citys;
    }


    public AdminCompanyWechat getAdminCompanyWechat(String subAppId) {
        AdminCompanyWechat one = adminCompanyWechatService.lambdaQuery().eq(AdminCompanyWechat::getSubAppId, subAppId).last("limit 1").one();
        return one;
    }

    /**
     * 附件归属绑定
     */
    public void bindFileFile(String objId, String objType, List<CommonFile> fileIds) {
        if (CollectionUtil.isEmpty(fileIds)) {
            return;
        }
        //先删除后绑定
        commonFileService.lambdaUpdate()
                .eq(CommonFile::getObjId, objId)
                .eq(CommonFile::getObjType, objType).remove();

        for (CommonFile file : fileIds) {
            file.setId(null);
            file.setObjId(objId);
            file.setObjType(objType);
        }
        commonFileService.saveBatch(fileIds);
    }

    /**
     * 查询附件列表
     */
    public List<CommonFile> queryFileByObjId(String objId, String objType) {
        return commonFileService.lambdaQuery()
                .eq(CommonFile::getObjId, objId)
                .eq(CommonFile::getObjType, objType)
                .orderByAsc(CommonFile::getCreateTime)
                .list();
    }

    /**
     * 查询附件列表
     */
    public List<String> queryFileUrlsByObjId(String objId, String objType) {
        List<CommonFile> commonFiles = this.queryFileByObjId(objId, objType);
        if (commonFiles.size() == 0) {
            return null;
        }
        return commonFiles.stream().map(CommonFile::getUrl).collect(Collectors.toList());
    }

    /**
     * 附件归属绑定
     */
    public void bindFile(String objId, String objType, List<String> fileIds) {
        //先删除后绑定
        commonFileService.lambdaUpdate()
                .eq(CommonFile::getObjId, objId)
                .eq(CommonFile::getObjType, objType)
                .notIn(CommonFile::getId, fileIds).remove();

        for (int i = 0; i < fileIds.size(); i++) {
            //这里分开每次存储的时候，每个图片创建时间加1秒，方便图片排序用
            commonFileService.lambdaUpdate()
                    .set(CommonFile::getObjId, objId)
                    .set(CommonFile::getObjType, objType)
                    .set(CommonFile::getCreateTime, DateUtils.addSeconds(new Date(), i))
                    .eq(CommonFile::getId, fileIds.get(i))
                    .update();
        }
    }

    /**
     * 获取当前用户
     */
    public CurrentCompanyWechat getCurrentCompanyWechat(HttpServletRequest request) {
        String appid = CommonUtils.getAPPID();
        String userId = CommonUtils.getUserId(request);

        if (com.aliyuncs.utils.StringUtils.isEmpty(appid)) {
            throw new RemoteServiceException("非法请求，缺少必须的参数");
        }
        AdminCompanyWechat adminCompanyWechat = adminCompanyWechatService.lambdaQuery()
                .eq(AdminCompanyWechat::getSubAppId, appid).one();
        if (adminCompanyWechat == null) {
            throw new RemoteServiceException("非法请求，请联系管理员配置必要的参数:" + appid);
        }

        User user = userService.getById(userId);

        CurrentCompanyWechat currentCompanyWechat = new CurrentCompanyWechat();
        BeanUtils.copyProperties(adminCompanyWechat, currentCompanyWechat);
        currentCompanyWechat.setCurrentCompanyWechatId(adminCompanyWechat.getCompanyWechatId());
        currentCompanyWechat.setCurrentCompanyName(adminCompanyWechat.getCompanyName());
        currentCompanyWechat.setUserId(CommonUtils.getUserId(request));
        currentCompanyWechat.setUser(user);
        return currentCompanyWechat;

    }

    /**
     * 查询街道列表
     */
    public List<Map> street(String province, String city, String area) {
        List<Map> street = getStreet(province, area);
        if (street.size() == 0) {
            Map map = new HashMap();
            map.put("name", area);
            street.add(map);
        }
        return street;
    }

    /**
     * 获取高德行政地址
     */
    public List<Map> getStreet(String province, String area) {
        try {
            String s = HttpUtil.get("https://restapi.amap.com/v3/config/district?keywords=" + province + "&subdistrict=1&key=" + mapWebKey);
            JSONObject jsonObject = JSONUtil.parseObj(s);
            String status = (String) jsonObject.get("status");
            if (status.equals("1")) {
                List<Object> districts = (List<Object>) jsonObject.get("districts");
                String adcode = (String) ((Map<String, Object>) districts.get(0)).get("adcode");

                String street = HttpUtil.get("https://restapi.amap.com/v3/config/district?keywords=" + area + "&subdistrict=1&key=" + mapWebKey + "&adcode=" + adcode);
                Object districts1 = JSONUtil.parseObj(street).get("districts");
                List<Map> maps = JSONUtil.toList(JSONUtil.toJsonStr(districts1), Map.class);

                List<Map> districts2 = JSONUtil.toList(JSONUtil.toJsonStr(maps.get(0).get("districts")), Map.class);
                return districts2;
            }
        } catch (Exception e) {
            return new ArrayList<>();
        }
        return new ArrayList<>();
    }

    /**
     * 微信scene
     */
    public String getSceneValue(String wxSceneId) {
        WxScene wxScene = this.getScene(wxSceneId);
        return wxScene == null ? "" : wxScene.getParam();
    }

    /**
     * 微信scene
     */
    public WxScene getScene(String wxSceneId) {
        WxScene wxScene = wxSceneService.getById(wxSceneId);
        if (!wxScene.getStatus()) {
            throw new RemoteServiceException("链接无效");
        }
        if (wxScene.getEndTime().getTime() < new Date().getTime()) {
            throw new RemoteServiceException("链接已过期");
        }
        return wxScene;
    }

    /**
     * 微信scene详情
     */
    public List<WxSceneDetail> getSceneDetail(String wxSceneId) {
        List<WxSceneDetail> list = wxSceneDetailService.lambdaQuery().eq(WxSceneDetail::getWxSceneId, wxSceneId).list();
        return list;
    }

    /**
     * 新增修改日志
     *
     * @param objId               各业务数据主键id
     * @param content             操作内容(例如：从 广州众炬 修改为 广州格力)
     * @param updateLogRemarkEnum 备注业务模块，自行增加枚举类型
     */
    public void addUpdateLog(String objId, String content, UpdateLogRemarkEnum updateLogRemarkEnum) {
        if (StringUtils.isBlank(content)) {
            return;
        }
        updateLogLogic.add(objId, "编辑", content, updateLogRemarkEnum);
    }

    public List<Region> queryRegionList(List<String> pid) {
        return regionService.lambdaQuery().in(Region::getPid, pid).list();
    }
}
